package handlers

import (
	"context"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/template_management"
)

// NewAccountTemplateGetHandler returns a AccountTemplateGetHandler.
func NewAccountTemplateGetHandler(db *sqlx.DB) *AccountTemplateGetHandler {
	return &AccountTemplateGetHandler{
		db: db,
	}
}

// AccountTemplateGetHandler handles GET /user.
type AccountTemplateGetHandler struct {
	db *sqlx.DB
}

func (h *AccountTemplateGetHandler) handle(
	ctx context.Context, account, name string,
) (*models.Template, error) {
	log := zerolog.Ctx(ctx)

	var templatedb models.TemplateDB

	sql := database.NewSQLHelper(ctx, h.db, *log)

	if err := sql.GetWhere(&templatedb, squirrel.Eq{
		models.TemplateDBAccountColumn: account,
		models.TemplateDBNameColumn:    name,
	}); err != nil {
		return nil, err
	}
	templatedb.FromDB()
	templatedb.ForceBase64()

	return &templatedb.Template, nil
}

// Handle handles the incoming request.
func (h *AccountTemplateGetHandler) Handle(
	params template_management.AccountTemplateGetParams, principal *models.Principal,
) middleware.Responder {
	if principal.IsAdmin {
		return template_management.
			NewAccountTemplateAddUnauthorized().
			WithPayload(&models.Error{
				Message: "Admin cannot manage templates. Please use a non-admin user",
			})
	}
	if principal.Subject != params.Account {
		return template_management.
			NewAccountTemplateAddUnauthorized().
			WithPayload(&models.Error{
				Message: "User can manage its own templates only",
			})
	}
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	template, err := h.handle(params.HTTPRequest.Context(), params.Account, params.Name)
	if err != nil {
		return InternalError(err, log, false)
	}

	return template_management.NewAccountTemplateGetOK().WithPayload(template)
}
